@@ -269,13 +269,25 @@ sint pairalign(sint istart, sint iend, sint jstart, sint jend)
     /* do pairwise alignment parallelly ... */
     /* Frist, sending all sequences to slaves ... */
 
+
+	/******* Note: Dec 18, 2005 ********************
+	 * If the number of slaves is greater than N*(N-1)/2, 
+	 * we will MPI_Send the sequence data to N*(N-1) slaves only.
+	 * This is to prevent the idled slaves not being able to get out of
+	 * their while(1) loop, see "parallel_compare.c". 
+	***********************************************/
+
 	mybsize = 0;
 	mybsize += (9+(nseqs+1)+(NUMRES)*(NUMRES))*sizeof(int) + 26*sizeof(char);
 	mybsize += sizeof(char)+2*sizeof(int)+4*sizeof(float);
 	for (i=1;i<=nseqs;i++)
 	   mybsize += (seqlen_array[i]+1);
 
-	for (pdest = 1; pdest < np ; pdest++ ){
+	if ((nseqs*(nseqs-1)/2)>=(np-1)) {
+	    for (pdest = 1; pdest < np ; pdest++ )
+    		MPI_Send(&mybsize, 1, MPI_INT, pdest, PAIRWISE_TAG, MPI_COMM_WORLD);
+	} else {
+	    for (pdest = 1; pdest <= (nseqs*(nseqs-1)/2) ; pdest++ )
     		MPI_Send(&mybsize, 1, MPI_INT, pdest, PAIRWISE_TAG, MPI_COMM_WORLD);
 	}
 
@@ -313,9 +325,16 @@ sint pairalign(sint istart, sint iend, sint jstart, sint jend)
 
 	wtime99 = MPI_Wtime();
 
-	for (pdest = 1; pdest < np ; pdest++ ){
+
+	if ((nseqs*(nseqs-1)/2)>=(np-1)) {
+	   for (pdest = 1; pdest < np ; pdest++ )
 	        MPI_Send(mpi_buffer, mybsize, MPI_PACKED, pdest, MY_DATA_TAG, MPI_COMM_WORLD);
 	}
+        else {
+	   for (pdest = 1; pdest <= (nseqs*(nseqs-1)/2); pdest++ )
+	        MPI_Send(mpi_buffer, mybsize, MPI_PACKED, pdest, MY_DATA_TAG, MPI_COMM_WORLD);
+	}
+
 	free(mpi_buffer);
 
 	wtime100 = MPI_Wtime();
